<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class FAI01400FaturamentoDeRefeicoesPorFilial extends \Zeedhi\Framework\Controller\Report {

	protected $qr2Strategy;
	protected $birtStrategy;
	protected $environment;
	protected $newCode;
	protected $connection;
	protected $crudService;

	public function __construct($reportService, $qr2Strategy, $birtStrategy, $environment, $newCode, $connection, $crudService) {
		$this->qr2Strategy  = $qr2Strategy;
		$this->birtStrategy = $birtStrategy;
		$this->environment  = $environment;
		$this->newCode      = $newCode;
		$this->connection  	= $connection;
		$this->crudService  = $crudService;

		parent::__construct($reportService);
	}

	protected $reportMapping = array (
		'FAI01400' =>
		array (
			'strategy' => 'BIRT',
			'parameterMapping' =>
			array (
				'DATAINI'    => 'P_DTINICIAL',
				'DATAFIN'    => 'P_DTFINAL',
				'NRAPURACGD' => 'P_NRAPURACGD',
				'CDCLIENTE'  => 'P_CDCLIENTE',
				'NRORG'      => 'P_NRORG'
			)
		),
		'FAI01401' =>
		array (
			'strategy' => 'BIRT',
			'parameterMapping' =>
			array (
				'CDOPERADOR'		=> 'P_CDOPERADOR',
				'DATAINICIAL'		=> 'P_DTINICIAL',
				'DATAFINAL' 		=> 'P_DTFINAL',
				'CDFILIAL'  		=> 'P_CDFILIAL',
				'IDCENTRAL' 		=> 'P_IDCENTRAL',
				'AGGREGATE_UNIT'	=>	'P_AGREGADAS',
				'CDCLIENTE' 		=> 'P_CDCLIENTE',
				'NRORG'				=> 'P_NRORG'
			)
		),
		'FAI01402' =>
		array (
			'strategy' => 'BIRT',
			'parameterMapping' =>
			array (
				'DATAINICIAL'		=> 'P_DTINICIAL',
				'DATAFINAL' 		=> 'P_DTFINAL',
				'CDFILIAL'  		=> 'P_CDFILIAL',
				'TCDFILIAL' 		=> 'P_TCDFILIAL',
				'CDCLIENTE' 		=> 'P_CDCLIENTE',
				'TCDCLIENTE'		=> 'P_TCDCLIENTE',
				'DATAINI'   		=> 'P_DTINI',
				'DATAFIN'   		=> 'P_DTFIN',
				'IDCENTRAL' 		=> 'P_IDCENTRAL',
				'AGGREGATE_UNIT'	=> 'P_AGREGADAS',
				'CDOPERADOR'		=> 'P_CDOPERADOR',
				'NRORG'     		=> 'P_NRORG'
			)
		),
		'FAI01403' =>
		array (
			'strategy' => 'BIRT',
			'parameterMapping' =>
			array (
				'DATAINICIAL'		=> 'P_DTINICIAL',
				'DATAFINAL' 		=> 'P_DTFINAL',
				'CDFILIAL'  		=> 'P_CDFILIAL',
				'CDCLIENTE' 		=> 'P_CDCLIENTE',
				'IDCENTRAL' 		=> 'P_IDCENTRAL',
				'AGGREGATE_UNIT'	=> 'P_AGREGADAS',
				'CDOPERADOR'		=> 'P_CDOPERADOR',
				'NRORG'     		=> 'P_NRORG'
			)
		)
	);

	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		$this->qr2Strategy->setQr2Type($row['QR2TYPE']);
		$this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));

		$row['CDOPERADOR'] = $this->environment->getCdOperador();
		$row['NRORG'] = $this->environment->getNrOrg();
		
		if($row['__report_name'] != 'FAI01400') {
			$row['TCDFILIAL'] = $row['ALL_CDFILIAL'] === "T" ? "T" : 'N';
			$row['TCDCLIENTE'] = $row['CLIENTE'] === 'T' ? "T" : 'N';
			$row['DATAINICIAL'] = $row['DATAINI'];
			$row['DATAFINAL'] = $row['DATAFIN'];
		}
		
		if(in_array($row['__report_name'], ['FAI01401', 'FAI01402', 'FAI01403'])){
			$row['CDFILIAL'] = $row['ALL_CDFILIAL'] ? 'T' : $row['CDFILIAL'];
			$row['CDCLIENTE'] = $row['CLIENTE'] == 'T' ? 'T' : $row['CDCLIENTE'];
		}
		$row['NRAPURACGD'] = 'T'.$this->newCode->getCodeByTableName('NRAPURACGD', $this->environment->getNrOrg(), 1, 9);
		$row['LISTA_FILIAIS'] = '';
		$row['IDCENTRAL'] = isset($row['CDFILIALCENTRAL']) && $row['CDFILIALCENTRAL'] ? 'S' : 'N';
		
		if($row['IDCENTRAL'] == 'N') {
			$row['AGGREGATE_UNIT'] = 'N';
		}
		
		if($row['IDCENTRAL'] == 'S'){
    	$this->handleCentralUnit($row);
    	$row['P_AGREGADAS'] = $row['AGGREGATE_UNIT'];
    	
		if(in_array($row['__report_name'], ['FAI01401', 'FAI01403'])) $row['CDFILIAL'] = $row['CDFILIALCENTRAL'];
			$this->executeProcedure($row, $row['CDFILIALCENTRAL'], $row['IDCENTRAL']);
		} else {
			if($row['ALL_CDFILIAL']){
				$filiais = $this->mountDataFilial($row);
				$row['LISTA_FILIAIS'] = $this->criarListaValores(implode('_', $filiais));
			} else {
				$row['LISTA_FILIAIS'] = $this->criarListaValores(implode('_',$row['CDFILIAL']));
				foreach ($row['CDFILIAL'] as $filial) {
					$this->executeProcedure($row, $filial);
				}
				$row['CDFILIAL'] = '_'.implode('_', $row['CDFILIAL']).'_';
			}
		}
	}
	
	private function handleCentralUnit(&$row) {
    if(isset($row['CDFILIALCENTRAL']) && $row['CDFILIALCENTRAL']){
			$row['CDFILIAL'] = $row['CDFILIALCENTRAL'];
			$row['AGGREGATE_UNIT'] = $row['ALL_AGGREGATE_UNITS'] === 'T'
				? $row['ALL_AGGREGATE_UNITS']
				: $this->criarListaValores(implode($row['AGGREGATE_UNIT'], '_'));
			
			if(in_array($row['__report_name'], ['FAI01402'])) {
				$row['TCDFILIAL'] = $row['AGGREGATE_UNIT'] === 'T' ? 'T' : 'N';
				$row['CDCLIENTE'] = $row['CLIENTE'] == 'T' ? 'T' : $row['CDCLIENTE'];
				$row['TCDCLIENTE'] = $row['CLIENTE'] == 'T' ? 'T' : 'N';
			}
    }
  }

	private function deleteTrabApuracGD($nrapuracentral, $cdfilial){
        $sql = <<<SQL
        DELETE FROM TRAB_APURAC_GD
                WHERE 
					  VALOR_NA_LISTA(CDFILIALCENTRAL, :P_CDFILIALCENTRAL) <> 0  AND
                      NRAPURACENTRAL  = :P_NRAPURACENTRAL   AND
                      NRORG           = :P_NRORG
SQL;
        $params = array(
            'P_CDFILIALCENTRAL' => $cdfilial,
            'P_NRAPURACENTRAL'  => $nrapuracentral,
            'P_NRORG'           => $this->environment->getNrOrg()
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

	public function criarListaValores($valores){
        if(strlen($valores) <= 4000){
            return $valores;
        }else{
            $lista = explode('_',$valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista,
            "NRORG" => $this->environment->getNrorgTrab()
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

	private function executeProcedure($row, $cdfilial, $P_CENTRAL = 'N'){
		$procedure = new StoredProcedure($this->connection, 'APURAC_RELGESDIRETA');
		
		$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial));
		$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row['DATAINI']));
		$procedure->addParam(new Param('P_DATAFIN', Param::PARAM_INPUT, $row['DATAFIN']));
		$procedure->addParam(new Param('P_NRAPURACGD', Param::PARAM_INPUT, $row['NRAPURACGD']));
		$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row['CDCLIENTE']));
		$procedure->addParam(new Param('P_EQUIVALENCIA', Param::PARAM_INPUT, 'N'));
		$procedure->addParam(new Param('P_CENTRAL', Param::PARAM_INPUT, $P_CENTRAL));
		$procedure->addParam(new Param('P_NRLISTA_AGREGADAS', Param::PARAM_INPUT, $row['AGGREGATE_UNIT']));
		$procedure->execute();
	}

	private function mountDataFilial($row){
		$conditions = array(
			array(
				"columnName" => "CDOPERADOR",
				"operator" => "=",
				"value" => $this->environment->getCdOperador()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('filioper', $conditions)->getRows();
		$filiais = array();
		foreach($dataSet as $filial){
			array_push($filiais, $filial['CDFILIAL']);
			$this->executeProcedure($row, $filial['CDFILIAL']);
		}
		return $filiais;
	}
}
